#version 430 core
/*
* Copyright 2019 Vienna University of Technology.
* Institute of Computer Graphics and Algorithms.
* This file is part of the ECG Lab Framework and must not be redistributed.
*/

in VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
} vert;

out vec4 color;

uniform vec3 camera_world;

uniform vec3 materialCoefficients; // x = ambient, y = diffuse, z = specular 
uniform float specularAlpha;
uniform sampler2D diffuseTexture;
uniform sampler2D StoneLightmap;
uniform sampler2D StartPlatform1Lightmap;
uniform sampler2D StartPlatform2Lightmap;
uniform sampler2D StartPlatform3Lightmap;
uniform sampler2D VikingRoomLightmap;
uniform sampler2D VulkanLightmap;
uniform sampler2D StaticRowPlatformsLightmap;
uniform sampler2D BigPlatform1Lightmap;
uniform sampler2D BigPlatform2Lightmap;
uniform sampler2D BigPlatform3Lightmap;
uniform sampler2D FlyPlatformLightmap;
uniform sampler2D ForestLightmap;

uniform int path;

uniform struct DirectionalLight {
	vec3 color;
	vec3 direction;
} dirL;

uniform struct PointLight {
	vec3 color;
	vec3 position;
	vec3 attenuation;
} pointL;


vec3 phong(vec3 n, vec3 l, vec3 v, vec3 diffuseC, float diffuseF, vec3 specularC, float specularF, float alpha, bool attenuate, vec3 attenuation) {
	float d = length(l);
	l = normalize(l);
	float att = 1.0;	
	if(attenuate) att = 1.0f / (attenuation.x + d * attenuation.y + d * d * attenuation.z);
	vec3 r = reflect(-l, n);
	return (diffuseF * diffuseC * max(0, dot(n, l)) + specularF * specularC * pow(max(0, dot(r, v)), alpha)) * att; 
}

void main() {	
	vec3 n = normalize(vert.normal_world);
	vec3 v = normalize(camera_world - vert.position_world);
	
	vec3 texColor = texture(diffuseTexture, vert.uv).rgb;
	vec3 lightmapColor = vec3(0, 1, 0);
	if (path == 0){
		lightmapColor = texColor;
	}
	else if(path == 1){
		lightmapColor = texture(StoneLightmap, vert.uv).rgb;
	}
	
	else if(path == 2){
		lightmapColor = texture(StartPlatform1Lightmap, vert.uv).rgb;
	}
	else if(path == 3){
		lightmapColor = texture(StartPlatform2Lightmap, vert.uv).rgb;
	}
	else if(path == 4){
		lightmapColor = texture(StartPlatform3Lightmap, vert.uv).rgb;
	}
	else if(path == 5){
		lightmapColor = texture(VikingRoomLightmap, vert.uv).rgb;
	}
	else if(path == 6){
		lightmapColor = texture(VulkanLightmap, vert.uv).rgb;
	}
	
	else if(path == 7){
		lightmapColor = texture(StaticRowPlatformsLightmap, vert.uv).rgb;
	}
	else if(path == 8){
		lightmapColor = texture(BigPlatform1Lightmap, vert.uv).rgb;
	}
	else if(path == 9){
		lightmapColor = texture(BigPlatform2Lightmap, vert.uv).rgb;
	}
	else if(path == 10){
		lightmapColor = texture(BigPlatform3Lightmap, vert.uv).rgb;
	}
	else if(path == 11){
		lightmapColor = texture(FlyPlatformLightmap, vert.uv).rgb;
	}
	else if(path == 12){
		lightmapColor = texture(ForestLightmap, vert.uv).rgb;
	}
	else{
		lightmapColor = vec3(0, 1, 0);
	}
	
	vec3 finalColor =  texColor * lightmapColor;
	color = vec4(finalColor, 1); // ambient
	
	
	// add directional light contribution
	//color.rgb += phong(n, -dirL.direction, v, dirL.color * texColor, materialCoefficients.y, dirL.color, materialCoefficients.z, specularAlpha, false, vec3(0));
			
	// add point light contribution
	color.rgb += phong(n, pointL.position - vert.position_world, v, pointL.color * texColor, materialCoefficients.y, pointL.color, materialCoefficients.z, specularAlpha, true, pointL.attenuation);
}

