#version 430 core
/*
* Copyright 2019 Vienna University of Technology.
* Institute of Computer Graphics and Algorithms.
* This file is part of the ECG Lab Framework and must not be redistributed.
*/

layout(location = 0) in vec3 position;
layout(location = 1) in vec3 normal;
layout(location = 2) in vec2 uv;

out VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
} vert;

uniform mat4 modelMatrix;
uniform mat4 viewProjMatrix;
uniform mat3 normalMatrix;
uniform float time;

float generateOffset(float x, float z){
	float radiansX = (x / 2.0f + time) * 2.0 * 3.14;
	float radiansZ = (z / 2.0f + time) * 2.0 * 3.14;
	return 0.1 * 0.5 * (sin(radiansZ) + cos(radiansX));
}

void main() {
	
	vert.normal_world = normalMatrix * normal;
	vert.uv = uv;
	vec4 position_world_ = modelMatrix * vec4(position, 1);
	float offset = generateOffset(position_world_.x, position_world_.z);
	position_world_.x += offset;
	position_world_.y += offset;
	position_world_.z += offset;
	vert.normal_world.x += offset;
	vert.normal_world.y += offset;
	vert.normal_world.z += offset;
	vert.position_world = position_world_.xyz;// + vec3(generateOffset(vert.position_world.x, vert.position_world.z), generateOffset(vert.position_world.x, vert.position_world.z), generateOffset(vert.position_world.x, vert.position_world.z));
	gl_Position = viewProjMatrix * position_world_;
}